import javafx.collections.ObservableList;
import javafx.geometry.Bounds;
import javafx.scene.Node;
import javafx.scene.image.Image;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.MeshView;
import javafx.scene.shape.TriangleMesh;
import javafx.scene.transform.Rotate;

/**
 * ビル
 */
public class PRSpriteBuilding extends PRSpriteT {

	private static final float[]	POINTS			= {
		-1, -2, -1,
		-1,  2, -1,
		 1,  2, -1,
		 1, -2, -1,
		 1, -2, -1,
		 1,  2, -1,
		 1,  2,  1,
		 1, -2,  1,
		-1, -2,  1,
		-1,  2,  1,
		-1,  2, -1,
		-1, -2, -1
	};

	private static final float[]	TEXCOORDS		= {
		0, 0,
		0, 1,
		1, 1,
		1, 0
	};

	private static final int[]		FACES			= {
		 0, 0,  1, 1,  2, 2,
		 2, 2,  3, 3,  0, 0,
		 4, 0,  5, 1,  6, 2,
		 6, 2,  7, 3,  4, 0,
		 8, 0,  9, 1, 10, 2,
		10, 2, 11, 3,  8, 0
	};

	/** イメージ */
	private static Image			image			= null;

	/** 回転（左） */
	private static final Rotate		ROTATE_LEFT		= new Rotate( 15, Rotate.Y_AXIS);

	/** 回転（右） */
	private static final Rotate		ROTATE_RIGHT	= new Rotate(-15, Rotate.Y_AXIS);

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			PRSpriteBuilding.image = PRImage.getImage("3d/image_e_town0103_s512_town0103_1.png");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param strPattern 配置パターン
	 * @param X補正位置
	 * @param pos 位置
	 */
	public PRSpriteBuilding(PRApplication application, String strPattern, double xHosei, int pos){

		// スーパークラスを呼び出す
		super(application);

		try{

			// メッシュビューを生成
			MeshView meshView = new MeshView();

			// メッシュを生成
			TriangleMesh mesh = new TriangleMesh();

			mesh.getPoints().addAll(POINTS);
			mesh.getTexCoords().addAll(TEXCOORDS);
			mesh.getFaces().addAll(FACES);

			// マテリアルを生成
			PhongMaterial material = new PhongMaterial();
			material.setDiffuseMap(image);

			// メッシュを登録
			meshView.setMesh(mesh);
			meshView.setMaterial(material);

			// メッシュビューを追加
			this.getChildren().add(meshView);

			double x = 0.0;

			// 配置パターン＝左の場合、
			if("左".equals(strPattern)){

				// 位置を補正
				x = -3.5 + xHosei;

				// 回転
				this.getTransforms().add(ROTATE_LEFT);

			// 配置パターン＝右の場合、
			}else if("右".equals(strPattern)){

				// 位置を補正
				x = 3.5 + xHosei;

				// 回転
				this.getTransforms().add(ROTATE_RIGHT);

			} // end if 配置パターン

			// X設定
			super.setTranslateX(x);

			// Z設定
			super.setTranslateZ(150);

			// Y設定
			Bounds bounds = this.getBoundsInParent();
			super.setTranslateY(-bounds.getHeight() / 2);

			// 速度設定
			mz = -1.0;

			// 影を設定
			application.addShadow(this, 0.6);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end priteBuilding

	/**
	 * 実行
	 */
	@Override
	public void run(){

		//
		// X方向
		//

		// 移動量取得
		double mx = -application.getPRSpritePrusher().mx;

		// 座標計算
		double x = super.getTranslateX();
		x = x + mx;

		// 移動
		super.setTranslateX(x);

		//
		// Z方向
		//

		// 座標取得
		double z = super.getTranslateZ();

		// 座標加算
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 0){
			// 削除する
			super.remove();
		}

		// 移動
		super.setTranslateZ(z);

		//
		// 当たり判定
		//

		// 境界を取得
		Bounds boundsThis = this.getBoundsInParent();

		// スプライト一覧を取得
		ObservableList<Node> nodeLst = application.getNodeLst();

		// スプライト一覧の件数を取得
		int countNodeLst = nodeLst.size();

		// スプライト一覧の件数分、ループ
		for(int i = 0; i < countNodeLst; i++){

			// スプライトを取得
			Node node = nodeLst.get(i);

			// 弾の場合、
			if(node instanceof PRSpriteTama){

				// 当たっている場合、
				if(boundsThis.intersects(node.getBoundsInParent())){

					// 弾へ変換
					PRSpriteTama spriteTama = (PRSpriteTama)node;

					// 跳ね返されている場合、
					if(spriteTama.isRapel()){
						// コンティニュー
						continue;
					}

					// 弾を跳ね返す
					spriteTama.rapel();

					// ループを抜ける
					break;

				} // end if 当たっている場合

			} // end if 弾の場合

		} // end for スプライト一覧の件数分、ループ

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.6;
	}

	@Override
	public double getHaniY() {
		return 0.8;
	}

}
