/**
 * マネル
 */
public class PRSpriteGroupManel extends PRSpriteGroup implements PRConstant {

	/** 配置パターン */
	private		String				strPattern		= null;

	/** ハリテ */
	private		PRSpriteHarite		spriteHarite	= null;

	/** 弾音フラグ */
	private		boolean				flgTama			= false;

	/** 残機 */
	private		int					remaining		= 0;

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param strParam 配置パターン
	 */
	public PRSpriteGroupManel(PRApplication application, String strParam){

		super(application);

		// パターンを保持
		this.strPattern = strParam;

		// パターン１ or パターン２（左から出現） or パターン３（右から出現） の場合、
		if("パターン１".equals(strPattern) || "パターン２".equals(strPattern) || "パターン３".equals(strPattern)){

			// 出現音スタート
			application.startWaveAppearanceManel();

		} // end if パターン

	} // end PRSpriteGroupManel

	/**
	 * コンストラクタ
	 * <pre>
	 * ボス用のコンストラクタです。
	 * </pre>
	 * @param application PRApplication
	 * @param strParam 配置パターン
	 * @param spriteHarite ハリテ
	 */
	public PRSpriteGroupManel(PRApplication application, String strParam, PRSpriteHarite spriteHarite){

		super(application);

		// パターンを保持
		this.strPattern = strParam;

		// ハリテを保持
		this.spriteHarite = spriteHarite;

		// ストーリーを停止する
		super.getPRApplication().getPRStoryPrusher().stop();

	} // end PRSpriteGroupManel

	/**
	 * 実行
	 * <pre>
	 * 当オブジェクトのふるまいを決定します。
	 * <pre>
	 */
	public void run(){

		// パターン１の場合、
		if("パターン１".equals(strPattern)){

			// 2.5秒、
			if(time <= 150){

				// 一定時間経過の場合、
				if((time % 20) == 0){

					// スプライトを生成
					PRSpriteManel spriteManel = new PRSpriteManel(super.getPRApplication(), this, strPattern);

					// スプライトを追加
					super.getPRApplication().addNode(spriteManel);

				} // end if 一定時間経過の場合

			// 5.0秒、
			}else if(time == 300){

				// スプライトグループを消す
				super.getPRApplication().removeSpriteGroup(this);

			} // end if 時間

		// パターン２（左から出現） or パターン３（右から出現） の場合、
		}else if("パターン２".equals(strPattern) || "パターン３".equals(strPattern)){

			// 2.5秒、
			if(time <= 150){

				// 一定時間経過の場合、
				if((time % 20) == 0){

					// スプライトを生成
					PRSpriteManel spriteManel = new PRSpriteManel(super.getPRApplication(), this, strPattern);

					// スプライトを追加
					super.getPRApplication().addNode(spriteManel);

				} // end if 一定時間経過の場合

			// 5.0秒、
			}else if(time == 300){

				// スプライトグループを消す
				super.getPRApplication().removeSpriteGroup(this);

			} // end if 時間

		// ボス１の場合、
		}else if("ボス１".equals(strPattern)){

			// 3.0秒
			if(time <= 180){

				// 一定時間経過の場合、
				if((time % 30) == 0){

					// ３体ループ
					for(int pos = 0; pos < 3; pos++){

						// スプライトを生成
						PRSpriteManel spriteManel = new PRSpriteManel(super.getPRApplication(), this, strPattern, pos);

						// スプライトを追加
						super.getPRApplication().addNode(spriteManel);

						// 残機＋１
						remaining++;

					} // end for ３体ループ

					// 出現音スタート
					application.startWaveAppearanceManel();

				} // end if 一定時間経過の場合

				// 3.0秒
				if(time == 180){

					// マネルBGMスタート
					application.startMP3BGMManel();

				}

			} // end if 時間

		} // end if パターン

	} // end run

	/**
	 * スプライト削除
	 * <pre>
	 * スプライトグループで管理しているスプライトを削除します。
	 * <pre>
	 */
	public void removeSprite(){

		// 残機−１
		remaining--;

		// 残機が無くなった場合、
		if(remaining == 0){

			// ハリテ退却
			spriteHarite.retreat();

			// ストーリーを再開する
			super.getPRApplication().getPRStoryPrusher().start();

			// マネルBGMストップ
			application.stopMP3BGMManel();

			// スプライトグループを消す
			super.getPRApplication().removeSpriteGroup(this);

		}

	} // end removeSprite

}
