import java.util.HashMap;

import javafx.collections.ObservableList;
import javafx.scene.Node;
import javafx.scene.Group;
import javafx.geometry.Bounds;
import javafx.scene.transform.Translate;
import javafx.scene.transform.Rotate;
import javafx.geometry.Point3D;

/**
 * 空母ハリテ
 * <pre>
 * 実は、ハリボテから命名しています。
 * 中身がない、ハリボテなので。
 * </pre>
 */
public class PRSpriteHarite extends PRSpriteT {

	/** 発射位置−左 */
	private static Point3D		POS_L					= new Point3D(-4.5, 0.0, 1.0);
	/** 発射位置−右 */
	private static Point3D		POS_R					= new Point3D( 4.5, 0.0, 1.0);

	/** フェーズ */
	private		int				phase					= 0;

	/** 時間 */
	private		int				time					= 0;

	/** パーツ */
	private		Group			groupHatch				= null;
	private		Group			groupBatteryL			= null;
	private		Group			groupBatteryR			= null;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteHarite.class, "3d/Harite.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param stage ステージ
	 */
	public PRSpriteHarite(PRApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト
			HashMap<String, Group> map = ObjReader.create(this);

			// パーツを保持
			groupHatch = map.get("Hatch");
			groupBatteryL = map.get("BatteryL");
			groupBatteryR = map.get("BatteryR");

			// ハッチ稼働部分位置移動
			// 回転軸に合わせて移動
			groupHatch.getTransforms().add(new Translate(0.0, -0.8, -0.1 - 0.5));
			// 元の位置へ移動
			groupHatch.setTranslateZ(0.1 + 0.5);
			groupHatch.setTranslateY(0.8);

			// ハッチの回転軸を設定
			groupHatch.setRotationAxis(Rotate.X_AXIS);

			// 位置設定
			super.setTranslateX(0.0);
			super.setTranslateY(-6.0);
			super.setTranslateZ(-10.0);

			// 速度設定
			mz = 0.5;

			// 影を設定
			application.addShadow(this, 6.0);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteHarite

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 時間＋１
		time++;

		// 座標取得
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// フェーズ＝０
		if(phase == 0){

			//
			// Z方向
			//

			// 座標加算
			z = z + mz;

			// 停止位置、
			if(z > 50){

				// 時間初期化
				time = 0;

				// フェーズ＝１
				phase = 1;

			}

			// 移動
			super.setTranslateZ(z);

		// フェーズ＝１
		}else if(phase == 1){

			// 0.5秒
			if(time == 60){

				// 時間初期化
				time = 0;

				// フェーズ＝２
				phase = 2;

			}

		// フェーズ＝２
		}else if(phase == 2){

			// ハッチを開く
			groupHatch.setRotate(time / 2);

			// 2.0秒
			if(time == 120){

				// 時間初期化
				time = 0;

				// フェーズ＝３
				phase = 3;

			}

		// フェーズ＝３
		}else if(phase == 3){

			// 1.0秒
			if(time < 60){

				// 何もしない

			// 1.0秒
			}else{

				// 時間初期化
				time = 0;

				// フェーズ＝４
				phase = 4;

			}

		// フェーズ＝４
		}else if(phase == 4){

			// マネルボスグループを生成
			PRSpriteGroupManel spriteGroup = new PRSpriteGroupManel(application, "ボス１", this);

			// アプリケーションにスプライトグループを追加
			application.addSpriteGroup(spriteGroup);

			// 時間初期化
			time = 0;

			// フェーズ＝５
			phase = 5;

		// フェーズ＝５
		}else if(phase == 5){

			// 2.0秒
			if(time < 120){

				// 何もしない

			// 3.0秒
			}else if(time < 180){

				// 0.2秒毎
				if(time % 12 == 0){

					// 敵の弾を生成
					PRSpriteTamaTeki1 spriteTamaTekiL = new PRSpriteTamaTeki1(application, this, POS_L, 0.8);
					// 敵の弾を追加
					application.addNode(spriteTamaTekiL);

					// 敵の弾を生成
					PRSpriteTamaTeki1 spriteTamaTekiR = new PRSpriteTamaTeki1(application, this, POS_R, 0.8);
					// 敵の弾を追加
					application.addNode(spriteTamaTekiR);

					// 敵の弾音スタート
					application.startWaveTamaTeki();

				}

			// 3.0秒
			}else{

				// 時間初期化
				time = 0;

			} // end if 時間

		// フェーズ＝６
		}else if(phase == 6){

			// 座標加算
			y = y + my;
			z = z + mz;

			// 画面の外にでた場合、
			if(z > 300){
				// 削除する
				super.remove();
			}

			// 移動
			super.setTranslateY(y);
			super.setTranslateZ(z);

		} // end if フェーズ

	} // end run

	/**
	 * 退却
	 */
	public void retreat(){

		// 時間初期化
		time = 0;

		// 速度設定
		my = -0.1;
		mz = 1.0;

		// フェーズ＝６
		phase = 6;

	} // end retreat

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.8;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 0.2;
	}

}
