import javafx.scene.Group;
import javafx.scene.paint.Color;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.Cylinder;
import javafx.scene.transform.Rotate;

/**
 * ゴルのレーザー（レーザー２）
 */
public class PRSpriteLaserTeki2 extends PRSpriteTamaTeki0 {

	/** レーザーの色 */
	private static final Color[]	COLOR_LASER		= new Color[]{
		Color.color(1.0, 0.8, 0.1, 0.8),
		Color.color(1.0, 0.6, 0.1, 0.6),
		Color.color(1.0, 0.4, 0.1, 0.4)
	};

	/** 敵スプライト */
	private		Group				sprite			= null;

	/** シリンダー */
	private		Cylinder			cylinder		= null;

	/** マテリアル */
	private		PhongMaterial		materail		= null;

	/** 時間 */
	private		int					time			= 0;

	/** 速度 */
	private		double				mx				= 0;
	private		double				my				= 0;
	private		double				mz				= 0;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param spritePrusher プレイヤー
	 */
	public PRSpriteLaserTeki2(PRApplication application, Group sprite, PRSpriteGroupLaserTeki2 spriteGroup, int time) {

		super(application);

		// 敵スプライトを保持
		this.sprite = sprite;

		// 時間を保持
		this.time = time;

		// シリンダーを生成
		cylinder = new Cylinder(0.25, 4.5, 12);

		// 回転させる（90度傾ける）
		cylinder.getTransforms().add(new Rotate(90, Rotate.X_AXIS));

		// シリンダーを追加
		this.getChildren().add(cylinder);

		// 座標
		double x = sprite.getTranslateX();
		double y = sprite.getTranslateY();
		double z = sprite.getTranslateZ();

		// マテリアルを生成
		materail = new PhongMaterial();

		// マテリアルを設定
		cylinder.setMaterial(materail);

		// 鏡面色を設定
		materail.setSpecularPower(1.0);

		// 影を設定
		application.addShadow(this, 0.5);

		// 速度を設定
		this.mx = spriteGroup.mx;
		this.my = spriteGroup.my;
		this.mz = spriteGroup.mz;

		// 回転させる（砲台分、傾ける）
		cylinder.getTransforms().add(new Rotate(spriteGroup.angleZ, Rotate.Z_AXIS));
		cylinder.getTransforms().add(new Rotate(spriteGroup.angleX, Rotate.X_AXIS));

		// 移動
		x = x + mx;
		y = y + my;
		z = z + mz;

		// 位置を設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// 砲台チャージ音スタート
		application.startWaveChargeBattleShip();

	} // end PRSpriteLaserTeki2

	/**
	 * 実行
	 */
	public void run(){

		// 時間設定
		time--;

		// 位置取得
		double x = super.getTranslateX();
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// 時間が０になるまで
		if(time > 0){

			// 砲台に合わせて移動
			x = sprite.getTranslateX();
			y = sprite.getTranslateY();
			z = sprite.getTranslateZ();

		// 上記以外の場合、
		}else{

			// 移動
			x = x + mx;
			y = y + my;
			z = z + mz;

		} // end if 時間

		// 位置設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// 画面の外にでた場合、
		if(z < 0){
			// 削除
			application.removeNode(this);
		}

		// 弾の色の設定
		if(time % 3 == 0){
			int indexColor = (int)(Math.random() * 3);
			materail.setDiffuseColor(COLOR_LASER[indexColor]);
			materail.setSpecularColor(COLOR_LASER[indexColor]);
		}

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Z
	 */
//	@Override
//	public double getHaniZ() {
//		return 2.0;
//	}

}
