import java.util.HashMap;

import javafx.geometry.Point3D;
import javafx.collections.ObservableList;
import javafx.scene.Node;
import javafx.scene.Group;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.shape.MeshView;
import javafx.geometry.Bounds;
import javafx.scene.paint.Color;

/**
 * マネル
 */
public class PRSpriteManel extends PRSpriteT {

	/** ボス１ */
	private final Color[]		COLORS = {
		Color.color(1.0, 0.4, 1.0),
		Color.color(1.0, 0.4, 0.8),
		Color.color(1.0, 0.4, 0.6),
		Color.color(1.0, 0.4, 0.6),
		Color.color(1.0, 0.4, 0.4),
		Color.color(1.0, 0.4, 0.4),
		Color.color(1.0, 0.4, 0.4)
	};

	/** スプライトグループ */
	private		PRSpriteGroupManel		spriteGroup				= null;

	/** 配置パターン */
	private		String					strPattern				= null;

	/** フェーズ */
	private		int						phase					= 0;

	/** 時間 */
	private 	int						time					= 0;

	/** 角度 */
	private		int						angle					= 0;

	/** 耐久度 */
	private 	int						life					= 2;

	/** 移動方向 */
	private		int						way						= 1;

	/** Y地面 */
	private		double					yJimen					= 0;

	/** 待ち時間 */
	private		int						timeWait				= 0;

	/** 敵の弾発射時間 */
	private		int						timeTamaTeki			= 240;

	/** 変色パーツ */
	private		Group					groupArmor				= null;
	private		Group					groupShoulder_Left		= null;
	private		Group					groupShoulder_Right		= null;

	/** 変色パーツのマテリアル */
	private		PhongMaterial			materailArmor			= null;
	private		PhongMaterial			materailShoulder_Left	= null;
	private		PhongMaterial			materailShoulder_Right	= null;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteManel.class, "3d/Manel.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * <pre>
	 * タイトル用のマネルになります。
	 * </pre>
	 * @param application PRApplication
	 * @param strParam 配置パターン
	 */
	public PRSpriteManel(PRApplication application){

		// スーパークラスを呼び出す
		super(application);

		try{

			// オブジェクト
			ObjReader.create(this);

			// 縮小する
			super.setScaleX(0.25);
			super.setScaleY(0.25);
			super.setScaleZ(0.25);

			// 座標設定
			super.setTranslateX(-0.8);
			super.setTranslateY(-this.getBoundsInParent().getHeight() / 2);
			super.setTranslateZ(6.2);

			// 回転軸
			Point3D p = new Point3D(0.0, 1.0, 0.0);

			// 回転軸を設定
			super.setRotationAxis(p);

			// 回転
			super.setRotate(-30);

			// 影を設定
			application.addShadow(this, 0.6);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteManel

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param spriteGroup スプライトグループ
	 * @param strParam 配置パターン
	 */
	public PRSpriteManel(PRApplication application, PRSpriteGroupManel spriteGroup, String strParam){

		// スーパークラスを呼び出す
		super(application);

		try{

			// スプライトグループを保持
			this.spriteGroup = spriteGroup;

			// パターンを保持
			this.strPattern = strParam;

			// オブジェクト
			ObjReader.create(this);

			// 縮小する
			super.setScaleX(0.25);
			super.setScaleY(0.25);
			super.setScaleZ(0.25);

			// パターン１の場合、
			if("パターン１".equals(strPattern)){

				// 座標設定
				super.setTranslateX(5.0);
				super.setTranslateY(-this.getBoundsInParent().getHeight() / 2);
				super.setTranslateZ(200);

				// 回転軸
				Point3D p = new Point3D(0.0, 1.0, 0.0);

				// 回転軸を設定
				super.setRotationAxis(p);

				// 速度
				mz = -0.8;

			// パターン２の場合、
			}else if("パターン２".equals(strPattern)){

				// 座標設定
				super.setTranslateX(-2.0);
				super.setTranslateY(-this.getBoundsInParent().getHeight() / 2);
				super.setTranslateZ(200);

				// 速度
				mz = -1.6;

			// パターン３の場合、
			}else if("パターン３".equals(strPattern)){

				// 座標設定
				super.setTranslateX(2.0);
				super.setTranslateY(-this.getBoundsInParent().getHeight() / 2);
				super.setTranslateZ(200);

				// 速度
				mz = -1.6;

			} // end if パターン

			// 影を設定
			application.addShadow(this, 0.6);

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteManel

	/**
	 * コンストラクタ
	 * <pre>
	 * ボス１パターンになります。
	 * </pre>
	 * @param application PRApplication
	 * @param spriteGroup スプライトグループ
	 * @param strParam 配置パターン
	 */
	public PRSpriteManel(PRApplication application, PRSpriteGroupManel spriteGroup, String strParam, int pos){

		// スーパークラスを呼び出す
		super(application);

		try{

			// スプライトグループを保持
			this.spriteGroup = spriteGroup;

			// パターンを保持
			this.strPattern = strParam;

			// オブジェクト
			HashMap<String, Group> map = ObjReader.create(this);

			// 縮小する
			super.setScaleX(0.25);
			super.setScaleY(0.25);
			super.setScaleZ(0.25);

			// 変色パーツを保持
			groupArmor = map.get("Armor");
			groupShoulder_Left = map.get("Shoulder");
			groupShoulder_Right = map.get("Shoulder.001");

			// 変色パーツのメッシュビューを取得
			MeshView meshViewArmor = (MeshView)(groupArmor.getChildren().get(0));
			MeshView meshViewShoulder_Left = (MeshView)(groupShoulder_Left.getChildren().get(0));
			MeshView meshViewShoulder_Right = (MeshView)(groupShoulder_Right.getChildren().get(0));

			// 変色パーツのマテリアルを取得
			materailArmor = (PhongMaterial)meshViewArmor.getMaterial();
			materailShoulder_Left = (PhongMaterial)meshViewShoulder_Left.getMaterial();
			materailShoulder_Right = (PhongMaterial)meshViewShoulder_Right.getMaterial();

			// 座標設定
			super.setTranslateX((pos - 1) * 1.5);
			super.setTranslateY(-7.0);
			super.setTranslateZ(50);

			// Y地面取得
			yJimen = -this.getBoundsInParent().getHeight() / 2;

			// 回転軸
			Point3D p = new Point3D(0.0, 1.0, 0.0);

			// 回転軸を設定
			super.setRotationAxis(p);

			// 影を設定
			application.addShadow(this, 0.6);

			// 速度
			my = 0.1;

			// 耐久度を設定
			life = 4;

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteManel

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 時間＋１
		this.time++;

		// 座標取得
		double x = super.getTranslateX();
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// パターン１の場合、
		if("パターン１".equals(strPattern)){

			// 移動用の時間を作成
			int timeMove = time % 120;

			// 移動中
			if(timeWait == 0){

				// X移動
				x = x + (0.3 - 0.25 * (200 - z) / 200) * way;

				// 角度
				angle = -45 * way;

				// 端まで来た場合、
				if(way == -1 && x < (-z / 40) ||
					way == 1 && x > ( z / 40)){

					// 移動方向を反対にする
					way = -way;

					// 待ち時間
					timeWait = 30;

					// 角度を戻す
					angle = 0;

				}

			// 待ちの場合、
			}else{

				// 待ち時間−１
				timeWait--;

			}

			// Z移動
			z = z + mz;

			// 画面の外にでた場合、
			if(z < 0){
				// 削除する
				super.remove();
			}

			// 移動
			super.setTranslateX(x);
			super.setTranslateZ(z);

			// 回転
			super.setRotate(angle);

			// 3.0秒
			if(time % 180 == 0){

				// 敵の弾を生成
				PRSpriteTamaTeki2 spriteTamaTeki2 = new PRSpriteTamaTeki2(application, this, 0.6);
				// 敵の弾を追加
				application.addNode(spriteTamaTeki2);

				// 弾音スタート
				application.startWaveTamaManel();

			}

		// パターン２ or パターン３ の場合、
		}else if("パターン２".equals(strPattern) || "パターン３".equals(strPattern)){

			// Z移動
			z = z + mz;

			// 画面の外にでた場合、
			if(z < 0){
				// 削除する
				super.remove();
			}

			// 移動
			super.setTranslateZ(z);

			// 1.5秒
			if(time % 90 == 0){

				// 敵の弾を生成
				PRSpriteTamaTeki2 spriteTamaTeki2 = new PRSpriteTamaTeki2(application, this, 0.4);
				// 敵の弾を追加
				application.addNode(spriteTamaTeki2);

				// 弾音スタート
				application.startWaveTamaManel();

			}

		// ボス１の場合、
		}else if("ボス１".equals(strPattern)){

			// フェーズ＝０
			if(phase == 0){

				// Z移動
				y = y + my;

				// 移動
				super.setTranslateY(y);

				// 地面に着いたら、
				if(y >= yJimen){

					// 地面に着地
					y = yJimen;

					// 移動
					super.setTranslateY(y);

					// 時間初期化
					time = 0;

					// フェーズ１へ
					phase = 1;

				}

			// フェーズ＝１
			}else if(phase == 1){

				// 1.0秒
				if(time < 60){

					// 何もしない

				// 1.5秒
				}else if(time < 90){

					// Z移動
					z = 40 + (90.0 - time) / 30 * 10;

					// 移動
					super.setTranslateZ(z);

				// 2.0秒
				}else{

					// 時間初期化
					time = 0;

					// フェーズ２へ
					phase = 2;

				}

			// フェーズ＝２
			}else if(phase == 2){

				// 1.0秒
				if(time < 60){

					// Z移動
					z = 10 + (60.0 - time) / 60 * 30;

					// 移動
					super.setTranslateZ(z);

				// 1.5秒
				}else if(time < 150){

					// 何もしない

				// 1.5秒
				}else{

					// 時間初期化
					time = 0;

					// フェーズ３へ
					phase = 3;

				}

			// フェーズ＝３
			}else if(phase == 3){

				// 2.0秒
				if(time < 120){

					// Y移動
					y = yJimen - 8 * Math.sin(time / 120.0 * 3.141592);

					// 移動
					super.setTranslateY(y);

					// Z移動
					z = 10 + 15 - 15 * Math.cos(time / 120.0 * 3.141592);

					// 移動
					super.setTranslateZ(z);

					// 1.25秒 or 1.5秒
					if(time == 75 || time == 90){

						// 敵の弾を生成
						PRSpriteTamaTeki2 spriteTamaTeki2 = new PRSpriteTamaTeki2(application, this, 0.4);
						// 敵の弾を追加
						application.addNode(spriteTamaTeki2);

						// 弾音スタート
						application.startWaveTamaManel();

					}

				// 3.0秒
				}else{

					// 時間初期化
					time = 0;

					// フェーズ２へ
					phase = 2;

				}

			} // end if フェーズ

			// 時間がきたら、
			if(time % 12 == 0){

				// ボス感を出す
				int indexColor = ((int)(Math.random() * 7));
				materailArmor.setDiffuseColor(COLORS[indexColor]);
				materailShoulder_Left.setDiffuseColor(COLORS[indexColor]);
				materailShoulder_Right.setDiffuseColor(COLORS[indexColor]);

			}

		} // end if パターン

		//
		// 当たり判定
		//

		// 爆発していない場合、
		if(!flgBomb){

			// 境界を取得
			Bounds boundsThis = this.getBoundsInParent();

			// スプライト一覧を取得
			ObservableList<Node> nodeLst = application.getNodeLst();

			// スプライト一覧の件数を取得
			int countNodeLst = nodeLst.size();

			// スプライト一覧の件数分、ループ
			for(int i = 0; i < countNodeLst; i++){

				// スプライトを取得
				Node node = nodeLst.get(i);

				// 弾の場合、
				if(node instanceof PRSpriteTama){

					// 当たっている場合、
					if(boundsThis.intersects(node.getBoundsInParent())){

						// 弾を削除
						application.removeNode(node);

						// 耐久力−１
						life--;

						// 耐久力がある場合、
						if(life >= 1){

							// 敵耐える音スタート
							application.startWaveTekiTaeru();

							// ループを抜ける
							break;

						}

						// 爆発
						flgBomb = true;

						// 爆発を生成
						PRSpriteBomb spriteBomb = new PRSpriteBomb(application, this, 0.04);

						// アプリケーションに爆発を追加
						application.addNode(spriteBomb);

						// 点数追加
						application.addScore(500);

						// 爆発音スタート
						application.startWaveBomb();

						// ボス１の場合、
						if("ボス１".equals(strPattern)){
							// スプライトグループに削除を通知
							spriteGroup.removeSprite();
						}

						// ループを抜ける
						break;

					} // end if 当たっている場合

				} // end if 弾の場合

			} // end for スプライト一覧の件数分、ループ

		} // end if 爆発していない場合

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.4;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 0.4;
	}

}
