import javafx.collections.ObservableList;
import javafx.geometry.Bounds;
import javafx.geometry.Point3D;
import javafx.scene.Node;

/**
 * レッド・アイ
 * <pre>
 * SKKから復活登場！
 * </pre>
 */
public class PRSpriteRedEye extends PRSpriteT {

	/** スプライトグループ */
	private		PRSpriteGroupRedEye		spriteGroup		= null;

	/** 配置パターン */
	private		String					strPattern		= null;

	/** 回転速度 */
	private		double					s				= 6;

	/** 時間 */
	private		int						time			= 0;

	/** 体力 */
	private		int						strength		= 4;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteRedEye.class, "3d/RedEye.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param spriteGroup スプライトグループ
	 * @param strPattern パターン
	 */
	public PRSpriteRedEye(PRApplication application, PRSpriteGroupRedEye spriteGroup, String strPattern){

		// スーパークラスを呼び出す
		super(application);

		try{

			// スプライトグループを保持
			this.spriteGroup = spriteGroup;

			// パターンを保持
			this.strPattern = strPattern;

			// Z設定
			super.setTranslateZ(100);

			// オブジェクト
			ObjReader.create(this);

			// 回転軸
			Point3D p = new Point3D(0.0, 1.0, 0.0);

			// 回転軸を設定
			super.setRotationAxis(p);

			// 速度設定
			mz = -0.6;

			// 影を設定
			application.addShadow(this, 0.6);

			// ボスの場合、
			if("ボス".equals(strPattern)){

				// 体力を6に上げる
				strength = 6;

			}

		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end PRSpriteRedEye

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 時間＋１
		time++;

		// 回転
		double angle = super.getRotate() % 360;
		angle = angle + this.s;
		super.setRotate(angle);

		// ザコパターン
		if("ザコ".equals(strPattern)){

			// 座標取得
			double z = super.getTranslateZ();

			// 2秒
			if(time < 120){

				// 座標加算
				z = z + mz;

				// 移動
				super.setTranslateZ(z);

			// 3秒・5.5秒・8秒・10.5秒
			}if(time == 180 || time == 330 || time == 480 || time == 630){

				// 敵のレーザーを生成
				PRSpriteLaserTeki1 spriteLaserTeki1 = new PRSpriteLaserTeki1(application, this, false);
				// 敵のレーザーを追加
				application.addNode(spriteLaserTeki1);

				// 敵のレーザー音１スタート
				application.startWaveLaser1Teki();

			// 13秒
			}else if(time < 780){

				// 何もしない

			// 7秒以上
			}else{

				// 座標加算
				z = z + mz;

				// 画面の外にでた場合、
				if(z < 0){

					// 削除する
					super.remove();
					// スプライトグループに削除を通知
					spriteGroup.removeSprite();

				}

				// 移動
				super.setTranslateZ(z);

			} // end if 時間

		// ボスパターン
		}else if("ボス".equals(strPattern)){

			// 座標取得
			double z = super.getTranslateZ();

			// 2.5秒
			if(time < 150){

				// 座標加算
				z = z + mz;

				// 移動
				super.setTranslateZ(z);

			// 2.0秒毎
			}if(time % 120 == 0){

				// 敵のレーザーを生成
				PRSpriteLaserTeki1 spriteLaserTeki1 = new PRSpriteLaserTeki1(application, this, true);
				// 敵のレーザーを追加
				application.addNode(spriteLaserTeki1);

				// 敵のレーザー音１スタート
				application.startWaveLaser1Teki();

			} // end if 時間

		} // end if パターン

		//
		// 当たり判定
		//

		// 爆発していない場合、
		if(!flgBomb){

			// 境界を取得
			Bounds boundsThis = this.getBoundsInParent();

			// スプライト一覧を取得
			ObservableList<Node> nodeLst = application.getNodeLst();

			// スプライト一覧の件数を取得
			int countNodeLst = nodeLst.size();

			// スプライト一覧の件数分、ループ
			for(int i = 0; i < countNodeLst; i++){

				// スプライトを取得
				Node node = nodeLst.get(i);

				// 弾の場合、
				if(node instanceof PRSpriteTama){

					// 当たっている場合、
					if(boundsThis.intersects(node.getBoundsInParent())){

						// 弾へ変換
						PRSpriteTama spriteTama = (PRSpriteTama)node;

						// 跳ね返されている場合、
						if(spriteTama.isRapel()){
							// コンティニュー
							continue;
						}

						// 角度が45〜135または225〜315の場合、
						if(angle > 45 && angle < 135 || angle > 225 && angle < 315){

							// 弾を跳ね返す
							spriteTama.rapel();

							// ループを抜ける
							break;

						// 上記以外の場合、
						}else{

							// 体力−１
							strength--;

							// 弾を削除
							application.removeNode(node);

							// 爆発音スタート
							application.startWaveBomb();

							// 体力が０の場合、
							if(strength <= 0){

								// 爆発
								flgBomb = true;

								// 爆発を生成
								PRSpriteBomb spriteBomb = new PRSpriteBomb(application, this, 0.05);

								// アプリケーションに爆発を追加
								application.addNode(spriteBomb);

								// 点数追加
								application.addScore(800);

								// スプライトグループに削除を通知
								spriteGroup.removeSprite();

							} // end if 体力が０より大きい場合

							// ループを抜ける
							break;

						} // end if

					} // end if 当たっている場合

				} // end if 弾の場合

			} // end for スプライト一覧の件数分、ループ

		} // end if 爆発していない場合

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 0.4;
	}

}
