import javafx.scene.Group;
import javafx.scene.shape.MeshView;
import javafx.scene.paint.PhongMaterial;
import javafx.scene.paint.Color;

/**
 * 敵の弾２
 */
public class PRSpriteTamaTeki2 extends PRSpriteTamaTeki0 {

	/** 時間 */
	private		int					time			= 0;

	/** 速度 */
	private		double				mx				= 0;
	private		double				my				= 0;
	private		double				mz				= 0;

	/**
	 * 初期化
	 */
	public static void init() {

		try{
			ObjReader.init(PRSpriteTamaTeki2.class, "3d/TamaTeki2.obj");
		}catch(Exception ex){
			ex.printStackTrace();
		}

	} // end init

	/**
	 * コンストラクタ
	 * @param application PRApplication
	 * @param spritePrusher プレイヤー
	 * @param speed 敵の弾のスピード
	 */
	public PRSpriteTamaTeki2(PRApplication application, PRSprite sprite, double speed) {

		super(application);

		// オブジェクト生成
		ObjReader.create(this);

		// 座標
		double x = sprite.getTranslateX();
		double y = sprite.getTranslateY();
		double z = sprite.getTranslateZ();

		// 位置を設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// グループ・メッシュビューを取得
		Group group = (Group)this.getChildren().get(0);
		MeshView meshView = (MeshView)(group.getChildren().get(0));

		// プレイヤ—を取得
		PRSpritePrusher spritePrusher = application.getPRSpritePrusher();

		// 座標の差を取得
		double saX = x - spritePrusher.getTranslateX();
		double saY = y - spritePrusher.getTranslateY();
		double saZ = z - spritePrusher.getTranslateZ();

		// Ｚ方向の分割数を求める
		double bunkatu = saZ / speed;

		// 速度を設定
		mz = -speed;
		mx = -saX / bunkatu;
		my = -saY / bunkatu;

		// 縮小する
		super.setScaleX(0.8);
		super.setScaleY(0.8);
		super.setScaleZ(0.8);

		// 影を設定
		application.addShadow(this);

	} // end PRSpriteTamaTeki2

	/**
	 * 実行
	 */
	@Override
	public void run(){

		// 位置取得
		double x = super.getTranslateX();
		double y = super.getTranslateY();
		double z = super.getTranslateZ();

		// 移動
		x = x + mx;
		y = y + my;
		z = z + mz;

		// 画面の外にでた場合、
		if(z < 0){

			// 削除
			application.removeNode(this);

		}

		// 位置設定
		super.setTranslateX(x);
		super.setTranslateY(y);
		super.setTranslateZ(z);

		// 時間設定
		time++;
		time = (time >= 7)?0: time;

		// 回転
		super.setRotate(super.getRotate() + 15);

	} // end run

	/**
	 * 当たり判定範囲X
	 */
	@Override
	public double getHaniX() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Y
	 */
	@Override
	public double getHaniY() {
		return 0.2;
	}

	/**
	 * 当たり判定範囲Z
	 */
	@Override
	public double getHaniZ() {
		return 2.0;
	}
}
